package es.upv.epsa.centro_veterinario.entidades;

import static es.upv.epsa.centro_veterinario.service.OfyService.ofy;
import es.upv.epsa.centro_veterinario.formularios.MascotaForm;

import com.google.api.server.spi.config.AnnotationBoolean;
import com.google.api.server.spi.config.ApiResourceProperty;
import com.google.common.base.Preconditions;
import com.googlecode.objectify.Key;
import com.googlecode.objectify.annotation.Cache;
import com.googlecode.objectify.annotation.Entity;
import com.googlecode.objectify.annotation.Id;
import com.googlecode.objectify.annotation.Index;
import com.googlecode.objectify.annotation.Parent;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Clase java para la entidad mascota
 * @author Ana Olcina
 * TFG 2017
 *
 */
@Entity
@Cache
public class Mascota {
	
    /**
     * El id será la clave del registro del datastore.
     * Se utiliza una asignación automática.
     */
    @Id
    private long id;

    /**
     * El nombre de la mascota.
     * Es un índice para poder indexar la propiedad y luego poder realizar búsquedas
     */
    @Index 
    private String nombre;

    /**
     * Atributo que almacenará la clave del perfil que es el antecesor de la mascota.
     */
    @Parent
    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    private Key<Perfil> perfilKey; 

    /**
     * Atributo que almacenará el nombre del dueño de la mascota.
     */
    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    private String duenoUserId;

    /**
     * Resto de datos de la mascota
     */
    private Date fecha_nacimiento;
    private String tipo_animal;
    private String raza;
    private String sexo;
    private String color;
    private Boolean esterilizado;
    private String num_microchip;
    private Date fecha_microchip;
    private String num_pasaporte;
    private String num_registro;
    private Date fecha_registro;
    private String num_poliza;
    private Date validez;

    /**
     * Constructor por defecto.
     */
    private Mascota() {}

    /**
     * Constructor público de la mascota
     * @param id
     * @param duenoUserId
     * @param mascotaForm
     */
    public Mascota(final long id, final String duenoUserId,
                      final MascotaForm mascotaForm) {
        Preconditions.checkNotNull(mascotaForm.getNombre(), "El nombre es requerido");
        this.id = id;
        this.perfilKey = Key.create(Perfil.class, duenoUserId);
        this.duenoUserId = duenoUserId;
        updateConMascotaForm(mascotaForm);
    }

    /**
     * Getters
     */
    public long getId() {
        return id;
    }

    public String getNombre() {
        return nombre;
    }
 
    public Date getFecha_nacimiento() {
		return fecha_nacimiento == null ? null : new Date(fecha_nacimiento.getTime());
	}

	public String getTipo_animal() {
		return tipo_animal;
	}

	public String getRaza() {
		return raza;
	}

	public String getSexo() {
        return sexo;
    }

	public String getColor() {
		return color;
	}

	public Boolean getEsterilizado() {
		return esterilizado;
	}

	public String getNum_microchip() {
		return num_microchip;
	}

	public Date getFecha_microchip() {
		return fecha_microchip  == null ? null : new Date(fecha_microchip.getTime());
	}

	public String getNum_pasaporte() {
		return num_pasaporte;
	}

	public String getNum_registro() {
		return num_registro;
	}

	public Date getFecha_registro() {
		return fecha_registro == null ? null : new Date(fecha_registro.getTime());
	}

	public String getNum_poliza() {
		return num_poliza;
	}

	public Date getValidez() {
		return validez  == null ? null : new Date(validez.getTime());
	}

	@ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    public Key<Perfil> getPerfilKey() {
        return perfilKey;
    }

    /**
     * Método que devuelve la clave de la mascota en formato String
     */
    public String getWebsafeMascotaKey() {
        return Key.create(perfilKey, Mascota.class, id).getString();
    }

    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    public String getDuenoUserId() {
        return duenoUserId;
    }

    /**
     * Para obtener el nombre del dueño de la mascota
     */
    public String getDuenoDisplayName() {
        Perfil dueno = ofy().load().key(getPerfilKey()).now();
        if (dueno == null) {
            return duenoUserId;
        } else {
            return dueno.getNombre();
        }
    }

    
    /**
     * Actualiza los datos de la mascota a través de los datos del formulario MascotaForm
     * @param mascotaForm
     */
     
    public void updateConMascotaForm(MascotaForm mascotaForm) {
    	final String SIN_DEFINIR = "No Definido este campo";
        
    	this.nombre = mascotaForm.getNombre();
        
        Date fecha_nacimiento = mascotaForm.getFecha_nacimiento();
        this.fecha_nacimiento = fecha_nacimiento == null ? null : new Date(fecha_nacimiento.getTime());

        String tipo_animal = mascotaForm.getTipo_animal();
        this.tipo_animal  = tipo_animal == null ? null : mascotaForm.getTipo_animal();
        
        String raza = mascotaForm.getRaza();
        this.raza = raza == null ? SIN_DEFINIR : mascotaForm.getRaza();
        
        String sexo = mascotaForm.getSexo();
        this.sexo = sexo == null ? null : mascotaForm.getSexo();
         
        String color = mascotaForm.getColor();
        this.color = color == null ? SIN_DEFINIR : mascotaForm.getColor();
        
        Boolean esterilizado = mascotaForm.getEsterilizado();
        this.esterilizado = esterilizado == null ? false : mascotaForm.getEsterilizado();
        
        String num_microchip = mascotaForm.getNum_microchip();
        this.num_microchip = num_microchip == null ? SIN_DEFINIR : mascotaForm.getNum_microchip();

        Date fecha_microchip = mascotaForm.getFecha_microchip();
        this.fecha_microchip = fecha_microchip == null ? null : new Date(fecha_microchip.getTime());

        String num_pasaporte = mascotaForm.getNum_pasaporte();
        this.num_pasaporte = num_pasaporte == null ? SIN_DEFINIR : mascotaForm.getNum_pasaporte();
        
        String num_registro = mascotaForm.getNum_registro();
        this.num_registro = num_registro == null ? SIN_DEFINIR : mascotaForm.getNum_registro();
     
        Date fecha_registro = mascotaForm.getFecha_registro();
        this.fecha_registro = fecha_registro == null ? null : new Date(fecha_registro.getTime());
        
        String num_poliza = mascotaForm.getNum_poliza();
        this.num_poliza = num_poliza == null ? SIN_DEFINIR : mascotaForm.getNum_poliza();
          
        Date validez = mascotaForm.getValidez();
        this.validez = validez == null ? null : new Date(validez.getTime());
  
    }
    
    /**
     * Actualización de los datos de la mascota
     * @param nombre
     * @param fecha_nacimiento
     * @param tipo_animal
     * @param raza
     * @param sexo
     * @param color
     * @param esterilizado
     * @param num_microchip
     * @param fecha_microchip
     * @param num_pasaporte
     * @param num_registro
     * @param fecha_registro
     * @param num_poliza
     * @param validez
     */
    public void update(String nombre, Date fecha_nacimiento, String tipo_animal, String raza,
    		String sexo, String color, Boolean esterilizado, String num_microchip, Date fecha_microchip, 
    		String num_pasaporte, String num_registro, Date fecha_registro, String num_poliza, Date validez) {
	       
    	this.nombre = nombre;
       
        if (fecha_nacimiento != null){
            this.fecha_nacimiento =  fecha_nacimiento;
        }else{
        	this.fecha_nacimiento = null;
        }
        if (tipo_animal != null) {
            this.tipo_animal  = tipo_animal;        	
        }
        if (raza != null){
            this.raza = raza;
        }
        if (sexo != null){
            this.sexo = sexo;       	
        }
        if (color != null){
            this.color = color;       	
        }
        if (esterilizado != null){
            this.esterilizado = esterilizado;        	
        }
  
        if (num_microchip != null){
            this.num_microchip = num_microchip;
        }

        if (fecha_microchip != null){
            this.fecha_microchip = fecha_microchip;
        }else{
        	this.fecha_microchip=null;
        }

        if (num_pasaporte != null){
            this.num_pasaporte = num_pasaporte;      	
        }

        if (num_registro != null){
            this.num_registro = num_registro;
        }

        if (fecha_registro != null){
            this.fecha_registro = fecha_registro;
        }else{
        	this.fecha_registro=null;
        }

        
        if (num_poliza != null){
            this.num_poliza = num_poliza;       	
        }
          
        if (validez != null){
            this.validez = validez;
        }else{
        	this.validez=null;
        }

  
    }
   
    /**
     * Método toString
     */

    @Override
    public String toString() {
    	DateFormat formatoFecha = new SimpleDateFormat("dd-MM-yyyy");
    	
		String dia;
        
    	StringBuilder stringBuilder = new StringBuilder("Id: " + id + "\n")
                .append("Nombre: ").append(nombre).append("\n");

        if (fecha_nacimiento != null) {
        	dia = formatoFecha.format(fecha_nacimiento);
        	stringBuilder.append("Fecha nacimiento: ").append(dia).append("\n");
        }

        if (raza != null) {
            stringBuilder.append("Raza: ").append(raza).append("\n");
        }
        
        if (sexo != null) {
            stringBuilder.append("Sexo: ").append(sexo).append("\n");
        }
        
        if (color != null) {
            stringBuilder.append("Color: ").append(color).append("\n");
        }
        if (tipo_animal != null) {
            stringBuilder.append("Tipo de animal: ").append(tipo_animal).append("\n");
        }
        if (esterilizado != null) {
        	if (esterilizado == false){
        		stringBuilder.append("Esterilizado: ").append("No").append("\n");
        	}else{
        		stringBuilder.append("Esterilizado: ").append("Sí").append("\n");
        	}
        }
        
        if (num_microchip != null) {
            stringBuilder.append("Número de microchip: ").append(num_microchip).append("\n");
        }
        
        if (fecha_microchip != null) {
        	dia = formatoFecha.format(fecha_microchip);
            stringBuilder.append("Fecha microchip: ").append(dia).append("\n");
        }
        
        if (num_pasaporte != null) {
            stringBuilder.append("Número de pasaporte: ").append(num_pasaporte).append("\n");
        }
        
        if (num_registro != null) {
            stringBuilder.append("Número de registro: ").append(num_registro).append("\n");
        }
        
        if (fecha_registro != null) {
        	dia = formatoFecha.format(fecha_registro);
            stringBuilder.append("Fecha de registro: ").append(dia).append("\n");
        }
        
        if (num_poliza != null) {
            stringBuilder.append("Poliza: ").append(num_poliza).append("\n");
        }
        
        if (validez != null) {
        	dia = formatoFecha.format(validez);
            stringBuilder.append("Válido hasta: ").append(dia).append("\n");
        }
        
        return stringBuilder.toString();
    }
    

}
