package es.upv.epsa.centro_veterinario.entidades;

import static es.upv.epsa.centro_veterinario.service.OfyService.ofy;
import es.upv.epsa.centro_veterinario.formularios.VisitaForm;

import com.googlecode.objectify.condition.IfNotDefault;
import com.google.api.server.spi.config.AnnotationBoolean;
import com.google.api.server.spi.config.ApiResourceProperty;
import com.google.common.base.Preconditions;
import com.googlecode.objectify.Key;
import com.googlecode.objectify.annotation.Cache;
import com.googlecode.objectify.annotation.Entity;
import com.googlecode.objectify.annotation.Id;
import com.googlecode.objectify.annotation.Index;
import com.googlecode.objectify.annotation.Parent;

import java.util.Date;

/**
 * Clase java para la entidad visita
 * @author Ana Olcina
 * TFG 2017
 *
 */

@Entity
@Cache
public class Visita {

    /**
     * El id será la clave del registro del datastore.
     * Se utiliza una asignación automática.
     */
    @Id
    private long id;

    /**
     * Fecha y hora de la visita actual
     */
    private Date fecha_visita;
    private String hora_visita;
    
    /**
     * Nombre de la mascota
     * Será un índice para realizar búsquedas
     */
    @Index
    private String nombre_mascota;
    
    /**
     * El veterinario que atienda a la visita estará indexado si no es un valor nulo
     */
    @Index(IfNotDefault.class)
    private String veterinario;

    /**
     * Atributo que almacenará la clave del perfil que es el antecesor de la visita.
     */
    @Parent
    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    private Key<Perfil> perfilKey; //la llave del perfil que es el antecesor de la visita

    /**
     * Atributo que almacenará el nombre del creador de la visita.
     */
    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    private String creadorUserId;

    /**
     * Resto de datos de la visita
     */
    private String centro_veterinario;
    private Double peso;
    private String vacunas;
    private String analisis;
    private String radiografias;
    private String exploracion;
    private String otras_observaciones;
    private Date proxima_visita;
    private String hora_prox_visita;

    /**
     * Constructor por defecto.
     */
    private Visita() {}

    /**
     * Constructor público de la visita
     * @param id
     * @param duenoUserId
     * @param visitaForm
     */
    public Visita(final long id, final String duenoUserId,
                      final VisitaForm visitaForm) {
        Preconditions.checkNotNull(visitaForm.getFecha_visita(), "La fecha es requerida");
        Preconditions.checkNotNull(visitaForm.getHora_visita(), "La hora es requerida");
        this.id = id;
        this.perfilKey = Key.create(Perfil.class, duenoUserId);
        this.creadorUserId = duenoUserId;
        updateConVisitaForm(visitaForm);
    }

    /**
     * Getters
     */
    
    public long getId() {
        return id;
    }
    
    public Date getFecha_visita() {
		return fecha_visita;
	}
    
    public String getHora_visita() {
		return hora_visita;
	}    

	public String getNombre_mascota() {
		return nombre_mascota;
	}
	
	public String getVeterinario() {
		return veterinario;
	}

	public String getCentro_veterinario() {
		return centro_veterinario;
	}

	public Double getPeso() {
		return peso;
	}

	public String getVacunas() {
		return vacunas;
	}

	public String getAnalisis() {
		return analisis;
	}

	public String getRadiografias() {
		return radiografias;
	}

	public String getExploracion() {
		return exploracion;
	}

	public String getOtras_observaciones() {
		return otras_observaciones;
	}
	 
	public Date getProxima_visita() {
		return proxima_visita;
	}
	
	public String getHora_prox_visita(){
		return hora_prox_visita;
	}

	@ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    public Key<Perfil> getPerfilKey() {
        return perfilKey;
    }

    /**
     * Método que devuelve la clave de la visita en formato String
     */
    public String getWebsafeVisitaKey() {
        return Key.create(perfilKey, Visita.class, id).getString();
    }

    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    public String getCreadorUserId() {
        return creadorUserId;
    }

    /**
     * Para obtener el nombre del creador de la visita
     */
    public String getCreadorDisplayName() {
        // Profile organizer = ofy().load().key(Key.create(Profile.class, organizerUserId)).now();
        Perfil creador = ofy().load().key(getPerfilKey()).now();
        if (creador == null) {
            return creadorUserId;
        } else {
            return creador.getNombre();
        }
    }

    
    /**
     * Actualiza los datos de la mascota a través de los datos del formulario MascotaForm
     * @param visitaForm
     */
    public void updateConVisitaForm(VisitaForm visitaForm) {
    	
        final String SIN_DEFINIR = "";
               
        Date fecha_visita = visitaForm.getFecha_visita();
        this.fecha_visita = new Date(fecha_visita.getTime());

        this.hora_visita = visitaForm.getHora_visita();

        String nombre_mascota  = visitaForm.getNombre_mascota();
        this.nombre_mascota  = nombre_mascota == null ? SIN_DEFINIR : visitaForm.getNombre_mascota();
        
        String veterinario = visitaForm.getVeterinario();
        this.veterinario = veterinario == null ? SIN_DEFINIR : visitaForm.getVeterinario();
        
        String centro_veterinario = visitaForm.getCentro_veterinario();
        this.centro_veterinario = centro_veterinario == null ? SIN_DEFINIR : visitaForm.getCentro_veterinario();
         
        Double peso = visitaForm.getPeso();
        this.peso = peso == null ? null : visitaForm.getPeso();
        
        String vacunas = visitaForm.getVacunas();
        this.vacunas = vacunas == null ? SIN_DEFINIR : visitaForm.getVacunas();
        
        String analisis = visitaForm.getAnalisis();
        this.analisis = analisis == null ? SIN_DEFINIR : visitaForm.getAnalisis();

        String radiografias = visitaForm.getRadiografias();
        this.radiografias = radiografias == null ? SIN_DEFINIR : visitaForm.getRadiografias();
        
        String exploracion = visitaForm.getExploracion();
        this.exploracion = exploracion == null ? SIN_DEFINIR : visitaForm.getExploracion();
     
        String otras_observaciones = visitaForm.getOtras_observaciones();
        this.otras_observaciones = otras_observaciones == null ? SIN_DEFINIR : visitaForm.getOtras_observaciones();
        
        Date proxima_visita = visitaForm.getProxima_visita();
        this.proxima_visita = proxima_visita == null ? null : new Date(proxima_visita.getTime());

        String hora_prox_visita = visitaForm.getHora_prox_visita(); 
        this.hora_prox_visita = hora_prox_visita == null ? null : visitaForm.getHora_prox_visita();;
 
    }
  
    /**
     * Actualización de los datos de la mascota
     * @param fecha_visita
     * @param hora_visita
     * @param nombre_mascota
     * @param veterinario
     * @param centro_veterinario
     * @param peso
     * @param vacunas
     * @param analisis
     * @param radiografias
     * @param exploracion
     * @param otras_observaciones
     * @param proxima_visita
     * @param hora_prox_visita
     */
    public void update(Date fecha_visita, String hora_visita, String nombre_mascota, String veterinario, String centro_veterinario,
    		Double peso, String vacunas, String analisis, String radiografias,
    		String exploracion, String otras_observaciones, Date proxima_visita, String hora_prox_visita) {
	       
    	this.fecha_visita = fecha_visita;
    	this.hora_visita =  hora_visita;
    	this.nombre_mascota = nombre_mascota;
       
        
        if (veterinario != null) {
            this.veterinario  = veterinario;        	
        }
        if (centro_veterinario != null){
            this.centro_veterinario = centro_veterinario;
        }
        if (peso != null){
            this.peso = peso;       	
        }
        if (vacunas != null){
            this.vacunas = vacunas;       	
        }
        if (analisis != null){
            this.analisis = analisis;        	
        }
  
        if (radiografias != null){
            this.radiografias = radiografias;
        }

        if (exploracion != null){
            this.exploracion = exploracion;
        }

        if (otras_observaciones != null){
            this.otras_observaciones = otras_observaciones;      	
        }

        if (proxima_visita != null){
            this.proxima_visita = proxima_visita;
        }else{
        	this.proxima_visita=null;
        }

        
        if (hora_prox_visita != null){
            this.hora_prox_visita = hora_prox_visita;
        }else{
        	this.hora_prox_visita=null;
        }
  
    }

    /**
     * Método toString()
     */
    @Override
    public String toString() {
        StringBuilder stringBuilder = new StringBuilder("Id: " + id + "\n")
                .append("Fecha: ").append(fecha_visita.toString()).append("\n")
                .append("Hora: ").append(hora_visita.toString()).append("\n");
       
        stringBuilder.append("Nombre Mascota: ").append(nombre_mascota).append("\n");
        
        if (veterinario != null) {
            stringBuilder.append("Veterinario: ").append(veterinario).append("\n");
        }
        if (peso != null) {
            stringBuilder.append("Peso: ").append(peso.toString()).append("\n");
        }
        if (vacunas != null) {
            stringBuilder.append("Vacunas: ").append(vacunas).append("\n");
        }
        if (analisis != null) {
            stringBuilder.append("Analisis: ").append(analisis).append("\n");
        }
        
        if (radiografias != null) {
            stringBuilder.append("Radiografias: ").append(radiografias).append("\n");
        }
        
        if (exploracion != null) {
            stringBuilder.append("Exploración: ").append(exploracion).append("\n");
        }
        
        if (otras_observaciones != null) {
            stringBuilder.append("Otras observaciones: ").append(otras_observaciones).append("\n");
        }
        
        if (proxima_visita != null) {
            stringBuilder.append("Próxima visita: ").append(proxima_visita.toString()).append("\n");
        }

        if (hora_prox_visita != null) {
            stringBuilder.append("Hora próxima visita: ").append(hora_prox_visita).append("\n");
        }

        return stringBuilder.toString();
    }

}
