package es.upv.epsa.centro_veterinario.formularios;

import static es.upv.epsa.centro_veterinario.service.OfyService.ofy;

import com.google.api.server.spi.config.AnnotationBoolean;
import com.google.api.server.spi.config.ApiResourceProperty;
import com.google.common.collect.ImmutableList;
import es.upv.epsa.centro_veterinario.entidades.Mascota;
import es.upv.epsa.centro_veterinario.entidades.Perfil;

import com.googlecode.objectify.Key;
import com.googlecode.objectify.cmd.Query;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;


/**
 * Clase simple de java (POJO) que representa las diversas opciones para realizar consultas de mascotas.
 * @author Ana Olcina
 * TFG 2017
 *
 */

public class MascotaConsultaForm {

    private static final Logger LOG = Logger.getLogger(MascotaConsultaForm.class.getName());

    /**
     * Enum que representa el tipo de campo a buscar.
     */
    public static enum FieldType {
        STRING
    }

    /**
     * Enum que representa los campos por los cuales se va a poder buscar.
     */
    public static enum Field {
    	
    	NOMBRE("nombre", FieldType.STRING);

        private String fieldName;

        private FieldType fieldType;

        private Field(String fieldName, FieldType fieldType) {
            this.fieldName = fieldName;
            this.fieldType = fieldType;
        }

        private String getFieldName() {
            return this.fieldName;
        }
    }

    /**
     * Enum que representa todos los operadores disponibles.
     */
    public static enum Operator {
        EQ("=="),
        LT("<"),
        GT(">"),
        LTEQ("<="),
        GTEQ(">="),
        NE("!=");

        private String queryOperator;

        private Operator(String queryOperator) {
            this.queryOperator = queryOperator;
        }

        private String getQueryOperator() {
            return this.queryOperator;
        }

        private boolean isInequalityFilter() {
            return this.queryOperator.contains("<") || this.queryOperator.contains(">") ||
                    this.queryOperator.contains("!");
        }
    }

    /**
     * Clase que representa una consulta para la consulta.
     */
    public static class Filter {
        private Field field;
        private Operator operator;
        private String value;

        public Filter () {}

        public Filter(Field field, Operator operator, String value) {
            this.field = field;
            this.operator = operator;
            this.value = value;
        }

        public Field getField() {
            return field;
        }

        public Operator getOperator() {
            return operator;
        }

        public String getValue() {
            return value;
        }
    }

    /**
     * Lista de los filtros de la consulta.
     */
    private List<Filter> filters = new ArrayList<>(0);

    /**
     * Para comprobar la viabilidad de la consulta
     */
    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    private Filter inequalityFilter;

    /**
     * Constructor por defecto
     */
    public MascotaConsultaForm() {}

    /**
     * Comprueba la viabilidad de toda la consulta.
     */
    private void checkFilters() {
        for (Filter filter : this.filters) {
            if (filter.operator.isInequalityFilter()) {
                // Sólo está permitido un filtro de desigualdad.
                if (inequalityFilter != null && !inequalityFilter.field.equals(filter.field)) {
                    throw new IllegalArgumentException(
                            "Sólo se puede aplicar un filtro de desigualdad en un campo.");
                }
                inequalityFilter = filter;
            }
        }
    }

    /**
     * Getter que devuelve la lista de filtros.
     */
    public List<Filter> getFilters() {
        return ImmutableList.copyOf(filters);
    }

    /**
     * Añade un filtro a la consulta
     * @param filter .
     */
    public MascotaConsultaForm filter(Filter filter) {
        if (filter.operator.isInequalityFilter()) {
        	// Sólo está permitido un filtro de desigualdad.
            if (inequalityFilter != null && !inequalityFilter.field.equals(filter.field)) {
                throw new IllegalArgumentException(
                        "Sólo se puede aplicar un filtro de desigualdad en un campo.");
            }
            inequalityFilter = filter;
        }
        filters.add(filter);
        return this;
    }

    /**
     * Ejecuta la consulta para los filtros especificados.
     * @return Query<Mascota>.
     */
    @ApiResourceProperty(ignored = AnnotationBoolean.TRUE)
    public Query<Mascota> getQuery(String userId) {
        
    	// Comprueba si los filtros son correctos.
        checkFilters();
        
        //Obtiene todas las mascotas del usuario
        Query<Mascota> query = ofy().load().type(Mascota.class)
                 .ancestor(Key.create(Perfil.class, userId));
        //Si no hay filtros de desigualdad
        if (inequalityFilter == null) {
            // Ordena por nombre.
            query = query.order("nombre");
        } else {
            // Si hay filtros de desigualdad, ordena por ese campo primero y luego por nombre.
            query = query.order(inequalityFilter.field.getFieldName());
            query = query.order("nombre");
        }
        
        //Empieza a filtrar los datos
        for (Filter filter : this.filters) {
            // Aplica los filtros en orden.
            if (filter.field.fieldType == FieldType.STRING) {
                query = query.filter(String.format("%s %s", filter.field.getFieldName(),
                        filter.operator.getQueryOperator()), filter.value);
            } 
        }
        //Añade la información al log
        LOG.info(query.toString());
        
        //Devuelve el resultado
        return query;
    }
}
