package es.upv.epsa.centro_veterinario.spi;

import static es.upv.epsa.centro_veterinario.service.OfyService.factory;
import static es.upv.epsa.centro_veterinario.service.OfyService.ofy;
import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiMethod.HttpMethod;
import com.google.api.server.spi.response.ConflictException;
import com.google.api.server.spi.response.ForbiddenException;
import com.google.api.server.spi.response.NotFoundException;
import com.google.api.server.spi.response.UnauthorizedException;
import com.google.appengine.api.taskqueue.Queue;
import com.google.appengine.api.taskqueue.QueueFactory;
import com.google.appengine.api.taskqueue.TaskOptions;
import com.google.appengine.api.users.User;

import es.upv.epsa.centro_veterinario.Constants;
import es.upv.epsa.centro_veterinario.entidades.AppEngineUser;
import es.upv.epsa.centro_veterinario.entidades.Mascota;
import es.upv.epsa.centro_veterinario.entidades.Perfil;
import es.upv.epsa.centro_veterinario.entidades.Visita;
import es.upv.epsa.centro_veterinario.formularios.MascotaForm;
import es.upv.epsa.centro_veterinario.formularios.MascotaConsultaForm;
import es.upv.epsa.centro_veterinario.formularios.VisitaForm;
import es.upv.epsa.centro_veterinario.formularios.VisitaConsultaForm;
import es.upv.epsa.centro_veterinario.formularios.PerfilForm;

import com.googlecode.objectify.Key;
import com.googlecode.objectify.Objectify;
import com.googlecode.objectify.Work;
import com.googlecode.objectify.cmd.Query;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.logging.Logger;
import javax.inject.Named;


/**
 * API del centro veterinario.
 * @author Ana Olcina
 * TFG 2017
 */


@Api(
        name = "veterinario",
        version = "v1",
        scopes = { Constants.EMAIL_SCOPE },
        clientIds = { Constants.WEB_CLIENT_ID, Constants.ANDROID_CLIENT_ID,
                Constants.API_EXPLORER_CLIENT_ID},
        audiences = {Constants.ANDROID_AUDIENCE},
        description = "API para la creación y visualización de mascotas y visitas," +
                " así como establecer el perfil del usuario"
)

public class VeterinarioApi {

    private static final Logger LOG = Logger.getLogger(VeterinarioApi.class.getName());
    
    /**
     * Método para obtener el perfil del usuario
     * @param user
     * @param userId
     * @return perfil
     */
    
    private static Perfil getPerfilDelUsuario(User user, String userId) {
        // Lo busca en el almacén de datos
        Perfil perfil = ofy().load().key(Key.create(Perfil.class, userId)).now();
        if (perfil == null) {
            
            String email = user.getEmail();
            // Crea un nuevo perfil si no existe.
            perfil = new Perfil(userId,
                    user.getNickname(), null, null, null, 
                    null, null, null, email);

        }
        return perfil;
    }

    /**
     * Solución para los ID nulos de usuario para los clientes de Android.
     */
    private static String getUserId(User user) {
        String userId = user.getUserId();
        if (userId == null) {
            LOG.info("userId es nulo, no se pueden obtener los datos.");
            AppEngineUser appEngineUser = new AppEngineUser(user);
            ofy().save().entity(appEngineUser).now();
            // Begin new session for not using session cache.
            Objectify objectify = ofy().factory().begin();
            AppEngineUser savedUser = objectify.load().key(appEngineUser.getKey()).now();
            userId = savedUser.getUser().getUserId();
            LOG.info("Identificador de usuario obtenido: " + userId);
        }
        return userId;
    }

    /**
     * Contenedor de tipo Booleano
     */
    public static class WrappedBoolean {

        private final Boolean result;

        public WrappedBoolean(Boolean result) {
            this.result = result;
        }

        public Boolean getResult() {
            return result;
        }
    }

    /**
     * Clase contenedora que puede adoptar el resultado genérico o algún tipo de excepción.
     * Se utiliza para los tipos de retorno de las transacciones de Objectify
     * @param <ResultType>
     */
    private static class TxResult<ResultType> {

        private ResultType result;

        private Throwable exception;

        private TxResult(ResultType result) {
            this.result = result;
        }

        private TxResult(Throwable exception) {
            if (exception instanceof NotFoundException ||
                    exception instanceof ForbiddenException ||
                    exception instanceof ConflictException) {
                this.exception = exception;
            } else {
                throw new IllegalArgumentException("Excepción no soportada.");
            }
        }

        private ResultType getResult() throws NotFoundException, ForbiddenException, ConflictException {
            if (exception instanceof NotFoundException) {
                throw (NotFoundException) exception;
            }
            if (exception instanceof ForbiddenException) {
                throw (ForbiddenException) exception;
            }
            if (exception instanceof ConflictException) {
                throw (ConflictException) exception;
            }
            return result;
        }
    }
    
     /**
      * Devuelve el perfil del usuario
      * @param user
      * @return
      * @throws UnauthorizedException
      */
    @ApiMethod(name = "getPerfil", path = "perfil", httpMethod = HttpMethod.GET)
    public Perfil getPerfil(final User user) throws UnauthorizedException {
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        return ofy().load().key(Key.create(Perfil.class, getUserId(user))).now();
    }

    /**
     * Guarda el perfil del usuario
     * @param user
     * @param perfilForm
     * @return
     * @throws UnauthorizedException
     */
    @ApiMethod(name = "guardarPerfil", path = "perfil", httpMethod = HttpMethod.POST)
    public Perfil guardarPerfil(final User user, final PerfilForm perfilForm)
            throws UnauthorizedException {
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
 
        String nombre = perfilForm.getNombre();
        String direccion = perfilForm.getDireccion();
        String poblacion = perfilForm.getPoblacion();
        String provincia = perfilForm.getProvincia();
        String codigo_postal = perfilForm.getCodigo_postal();
        String telefono = perfilForm.getTelefono();
        String movil = perfilForm.getMovil();

        Perfil perfil = ofy().load().key(Key.create(Perfil.class, getUserId(user))).now();
        
        if (perfil == null) {
            
            if (nombre == null) {
            	nombre = user.getNickname();
            }
  
            perfil = new Perfil(getUserId(user), nombre, direccion, poblacion,
        			provincia, codigo_postal, telefono, movil, user.getEmail());
        } else {
            perfil.update(nombre, direccion, poblacion,
        			provincia, codigo_postal, telefono, movil);
        }
        ofy().save().entity(perfil).now();
        return perfil;
    }
    
  
  @ApiMethod(
            name = "consultaUsuario",
            path = "consultaUsuario",
            httpMethod = HttpMethod.POST
    )
    public List<String> consultaUsuario(final User user) throws UnauthorizedException{
    	
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        String nombre = user.getNickname();

        List<String> nombreUsuario = new ArrayList<>(0);
        int index=0;
        
        nombreUsuario.add(index, nombre);

        return nombreUsuario;

        
    }

    /**
     * Crea una mascota y la almacena en el datastore
     * Envía el email al usuario con los datos proporcionados
     * @param user
     * @param mascotaForm
     * @return
     * @throws UnauthorizedException
     */
    @ApiMethod(name = "crearMascota", path = "mascota", httpMethod = HttpMethod.POST)
    public Mascota crearMascota(final User user, final MascotaForm mascotaForm)
        throws UnauthorizedException {
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }

        Key<Perfil> perfilKey = Key.create(Perfil.class, getUserId(user));
        final Key<Mascota> mascotaKey = factory().allocateId(perfilKey, Mascota.class);
        final long mascotaId = mascotaKey.getId();
        final Queue queue = QueueFactory.getDefaultQueue();
        final String userId = getUserId(user);

        Mascota mascota = ofy().transact(new Work<Mascota>() {
            @Override
            public Mascota run() {

                Perfil perfil = getPerfilDelUsuario(user, userId);
                Mascota mascota = new Mascota(mascotaId, userId, mascotaForm);

                ofy().save().entities(mascota, perfil).now();
                
                queue.add(ofy().getTransaction(),
                        TaskOptions.Builder.withUrl("/tasks/send_confirmation_email")
                        .param("email", perfil.getEmail())
                        .param("mascotaInfo", mascota.toString()));
                return mascota;
            }
        });
        return mascota;
    }

    /**
     * Actualiza una mascota
     * @param user
     * @param mascotaForm
     * @param websafeMascotaKey
     * @return
     * @throws UnauthorizedException
     * @throws NotFoundException
     * @throws ForbiddenException
     * @throws ConflictException
     */
    @ApiMethod(
            name = "updateMascota",
            path = "mascota/{websafeMascotaKey}",
            httpMethod = HttpMethod.PUT
    )
    public Mascota updateMascota(final User user, final MascotaForm mascotaForm,
                                       @Named("websafeMascotaKey")
                                       final String websafeMascotaKey)
            throws UnauthorizedException, NotFoundException, ForbiddenException, ConflictException {

        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        final String userId = getUserId(user);

        TxResult<Mascota> result = ofy().transact(new Work<TxResult<Mascota>>() {
            @Override
            public TxResult<Mascota> run() {

                Key<Mascota> mascotaKey = Key.create(websafeMascotaKey);
                Mascota mascota = ofy().load().key(mascotaKey).now();
                if (mascota == null) {
                    return new TxResult<>(
                            new NotFoundException("No se ha encontrado una mascota con la clave: "
                                    + websafeMascotaKey));
                }
                // If the user is not the owner, throw a 403 error.
                Perfil perfil = ofy().load().key(Key.create(Perfil.class, userId)).now();
                if (perfil == null || !mascota.getDuenoUserId().equals(userId)) {
                    return new TxResult<>(
                            new ForbiddenException("Sólo el dueño puede modificar la información de la mascota."));
                }

                String nombre = mascotaForm.getNombre();
            	Date fecha_nacimiento = null;
            	if (mascotaForm.getFecha_nacimiento() != null){
            		fecha_nacimiento = mascotaForm.getFecha_nacimiento();
            	}else{
            		fecha_nacimiento = null;
            	}
                String tipo_animal = mascotaForm.getTipo_animal();
                String raza = mascotaForm.getRaza();
                String sexo = mascotaForm.getSexo();
                String color = mascotaForm.getColor();
                Boolean esterilizado = mascotaForm.getEsterilizado();
                String num_microchip = mascotaForm.getNum_microchip();
                Date fecha_microchip = null;
                if (mascotaForm.getFecha_microchip() != null){
                	fecha_microchip = mascotaForm.getFecha_microchip();
            	}else{
            		fecha_microchip = null;
            	}
                String num_pasaporte = mascotaForm.getNum_pasaporte();
                String num_registro = mascotaForm.getNum_registro();
                Date fecha_registro = null;
                if (mascotaForm.getFecha_registro() != null){
                	fecha_registro = mascotaForm.getFecha_registro();
            	}else{
            		fecha_registro = null;
            	}
                String num_poliza = mascotaForm.getNum_poliza();  
                Date validez = null;
                if (mascotaForm.getValidez() != null){
            		validez = mascotaForm.getValidez();
            	}else{
            		validez = null;
            	}
               
                mascota.update(nombre, fecha_nacimiento, tipo_animal, raza, sexo, color, esterilizado,
                		num_microchip, fecha_microchip, num_pasaporte, num_registro, fecha_registro, 
                		num_poliza, validez);
                ofy().save().entity(mascota).now();
                return new TxResult<>(mascota);
            }
        });
        return result.getResult();
    }

     /**
      * Devuelve una mascota a través de su id
      * @param websafeMascotaKey
      * @return
      * @throws NotFoundException
      */
    @ApiMethod(
            name = "getMascota",
            path = "mascota/{websafeMascotaKey}",
            httpMethod = HttpMethod.GET
    )
    public Mascota getMascota(
            @Named("websafeMascotaKey") final String websafeMascotaKey)
            throws NotFoundException {
        Key<Mascota> mascotaKey = Key.create(websafeMascotaKey);
        Mascota mascota = ofy().load().key(mascotaKey).now();
        if (mascota == null) {
            throw new NotFoundException("No se encuentra la masscota con la clave: " + websafeMascotaKey);
        }
        return mascota;
    }

    /**
     * Devuelve las mascotas del usuario
     * @param user
     * @return
     * @throws UnauthorizedException
     * @throws NotFoundException
     */
    @ApiMethod(
            name = "getMascotasDelUsuario",
            path = "getMascotasDelUsuario",
            httpMethod = HttpMethod.GET
    )
    public Collection<Mascota> getMascotasDelUsuario(final User user)
            throws UnauthorizedException, NotFoundException {
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        Perfil perfil = ofy().load().key(Key.create(Perfil.class, getUserId(user))).now();
        if (perfil == null) {
            throw new NotFoundException("El perfil no existe.");
        }
        List<String> keyListaPropias = perfil.getMascotasPropiasKeys();
        List<Key<Mascota>> keysPropias = new ArrayList<>();
        for (String keyString : keyListaPropias) {
        	keysPropias.add(Key.<Mascota>create(keyString));
        }
        return ofy().load().keys(keysPropias).values();
    }

    /**
     * Consulta las mascotas
     * @param mascotaConsultaForm
     * @param user
     * @return
     * @throws UnauthorizedException
     */

    @ApiMethod(
            name = "consultaMascotas",
            path = "consultaMascotas",
            httpMethod = HttpMethod.POST
    )
    public List<Mascota> consultaMascotas(MascotaConsultaForm mascotaConsultaForm, final User user) throws UnauthorizedException {
    	   
    	if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        String userId = getUserId(user);
        
        Query<Mascota> consulta = mascotaConsultaForm.getQuery(userId);
    
   	
        return consulta.list();
        
        
    }
    
    
    
    
   

    /**
     * Devuelve únicamente el nombre de las mascotas creadas por un usuario
     * @param user
     * @return
     * @throws UnauthorizedException
     */
    @ApiMethod(
            name = "getNombreMascotasCreadas",
            path = "getNombreMascotasCreadas",
            httpMethod = HttpMethod.POST
    )
    public List<String> getNombreMascotasCreadas(final User user) throws UnauthorizedException {

        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        String userId = getUserId(user);
        List<Mascota> lista = ofy().load().type(Mascota.class)
                .ancestor(Key.create(Perfil.class, userId))
                .list();
        List<String> nombresMascotas = new ArrayList<>(0);
        int index=0;
        for (Mascota mascota: lista){
        	nombresMascotas.add(index, lista.get(index).getNombre().toString());
        	index++;
        }
        return nombresMascotas;
    }
    
    /**
     * Devuelve las mascotas creadas por el usuario
     * @param user
     * @return
     * @throws UnauthorizedException
     */
    @ApiMethod(
            name = "getMascotasCreadas",
            path = "getMascotasCreadas",
            httpMethod = HttpMethod.POST
    )
    public List<Mascota> getMascotasCreadas(final User user) throws UnauthorizedException {
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        String userId = getUserId(user);
        
        
        return ofy().load().type(Mascota.class)
                .ancestor(Key.create(Perfil.class, userId))
                .list();
    }

   

    /**
     * Elimina una mascota
     * @param user
     * @param websafeMascotaKey
     * @return
     * @throws UnauthorizedException
     * @throws NotFoundException
     * @throws ForbiddenException
     * @throws ConflictException
     */
    @ApiMethod(
            name = "eliminarMascota",
            path = "mascota/{websafeMascotaKey}/registros",
            httpMethod = HttpMethod.DELETE
    )
    public WrappedBoolean eliminarMascota(final User user,
                                            @Named("websafeMascotaKey")
                                            final String websafeMascotaKey)
            throws UnauthorizedException, NotFoundException, ForbiddenException, ConflictException {
        if (user == null) {
            throw new UnauthorizedException("Autorización Requerida");
        }
        final String userId = getUserId(user);
        TxResult<Boolean> result = ofy().transact(new Work<TxResult<Boolean>>() {

        	@Override
            public TxResult<Boolean> run() {
                Key<Mascota> mascotaKey = Key.create(websafeMascotaKey);
                Mascota mascota = ofy().load().key(mascotaKey).now();
                if (mascota == null) {
                    return new TxResult<>(new NotFoundException(
                            "Mascota con encontrada con la clave: " + websafeMascotaKey));
                }else{
                    ofy().delete().entity(mascota).now();
                    return new TxResult<>(true);
                }
   
        	}   
        	});

	        return new WrappedBoolean(result.getResult());
    }
    
    /**
     * Crea una visita y la almacena en el datastore.
     * Envía el correo al usuario con la fecha y hora de la próxima visita si ha proporcionado dicha información
     * @param user
     * @param visitaForm
     * @return
     * @throws UnauthorizedException
     */

    @ApiMethod(name = "crearVisita", path = "visita", httpMethod = HttpMethod.POST)
    public Visita crearVisita(final User user, final VisitaForm visitaForm)
        throws UnauthorizedException {
    	
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }

        Key<Perfil> perfilKey = Key.create(Perfil.class, getUserId(user));
        final Key<Visita> visitaKey = factory().allocateId(perfilKey, Visita.class);
        final long visitaId = visitaKey.getId();
        final Queue queue = QueueFactory.getDefaultQueue();
        final String userId = getUserId(user);

        Visita visita = ofy().transact(new Work<Visita>() {
            @Override
            public Visita run() {

                Perfil perfil = getPerfilDelUsuario(user, userId);
                Visita visita = new Visita(visitaId, userId, visitaForm);

                ofy().save().entities(visita, perfil).now();
                if (visita.getProxima_visita()!= null){
                	String nom_usuario = perfil.getNombre();
            		String nom_mascota = visita.getNombre_mascota();
                    Date fecha = visita.getProxima_visita();
                    DateFormat formatoFecha = new SimpleDateFormat("dd-MM-yyyy");
                    DateFormat formatoFecha2 = new SimpleDateFormat("yyyyMMdd");
            		String dia = formatoFecha.format(visita.getProxima_visita());
            		String dia_cita = formatoFecha2.format(visita.getProxima_visita());
            		String fecha_cita;
            		String visitaInfo; 

            		if (visita.getHora_prox_visita() != null){
            			String hora = visita.getHora_prox_visita();
                		String hora2 = visita.getHora_prox_visita(); 
                		String hora3 = hora2.substring(0,2);
                		int horaNum = Integer.parseInt(hora3)+1;
                		String horaNew = null;
                		if (horaNum <=9){
                			horaNew = "0"+Integer.toString(horaNum);
                		}else{
                			horaNew = Integer.toString(horaNum);
                		}
                		 
                		hora2 = hora2.replace(hora3, horaNew);
                		String horaInicio = hora.replaceAll(":", "");
              			String horaFin = hora2.replaceAll(":", "");
              			visitaInfo="La próxima visita de "+nom_mascota+ 
                				" al veterinario, será el día: "+dia+" a las: "+hora+"h.";
            			fecha_cita=dia_cita+"T"+horaInicio+"00/"+dia_cita+"T"+horaFin+"00";
            		}else{
            			visitaInfo="La próxima visita de "+nom_mascota+ 
                				" al veterinario, será el día: "+dia+" pero aún no hemos concretado la hora.";
            			fecha_cita=dia_cita+"/"+dia_cita;
            		}
          		
            		String asunto_cita="Cita+veterniario+"+nom_mascota;

                    String desc_cita="Siguiente+visita+de "+nom_mascota;
                    
                    String centro_vet=visita.getCentro_veterinario();
                    
                    String html = "<h3>Próxima visita veterinaria</h3>"
                    		+"<p>Hola "+nom_usuario+", <br>"+visitaInfo+"</p>"
                    		+"<p>Puedes añadir esta cita a tu calendario de Google pulsando el siguiente botón.</p>"
                    		+"<a href='https://calendar.google.com/calendar/render?action=TEMPLATE&text="
                    		+asunto_cita+"&dates="+fecha_cita+"&details="+desc_cita+"&location="+centro_vet+"&trp=false'><img src='http://www.google.com/calendar/images/ext/gc_button6_es.gif' border='0'></a>";
            		
                     queue.add(ofy().getTransaction(),
                            TaskOptions.Builder.withUrl("/tasks/send_visit_email")
                            .param("email", perfil.getEmail()) 						
                            .param("visitaInfo", visitaInfo)
                            .param("mascota", nom_mascota)
                            .param("centro", visita.getCentro_veterinario())
                    		.param("html_texto", html));
                }

                return visita;
            }
        });
        return visita;
    }

    /**
     * Actualiza una visita
     * Envía el correo al usuario con la fecha y hora de la próxima visita si ha proporcionado dicha información
     * @param user
     * @param visitaForm
     * @param websafeVisitaKey
     * @return
     * @throws UnauthorizedException
     * @throws NotFoundException
     * @throws ForbiddenException
     * @throws ConflictException
     */
    @ApiMethod(
            name = "updateVisita",
            path = "visita/{websafeVisitaKey}",
            httpMethod = HttpMethod.PUT
    )
    public Visita updateVisita(final User user, final VisitaForm visitaForm,
                                       @Named("websafeVisitaKey")
                                       final String websafeVisitaKey)
            throws UnauthorizedException, NotFoundException, ForbiddenException, ConflictException {

        final Queue queue = QueueFactory.getDefaultQueue();
    	if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        final String userId = getUserId(user);

        TxResult<Visita> result = ofy().transact(new Work<TxResult<Visita>>() {
            @Override
            public TxResult<Visita> run() {
                Key<Visita> visitaKey = Key.create(websafeVisitaKey);
                Visita visita = ofy().load().key(visitaKey).now();
                if (visita == null) {
                    return new TxResult<>(
                            new NotFoundException("No se ha encontrado una visita con la clave: "
                                    + websafeVisitaKey));
                }
                Perfil perfil = ofy().load().key(Key.create(Perfil.class, userId)).now();
                if (perfil == null || !visita.getCreadorUserId().equals(userId)) {
                    return new TxResult<>(
                            new ForbiddenException("Sólo el creador puede modificar la información de la visita."));
                }
                
                Date fecha_visita = visitaForm.getFecha_visita();
                String hora_visita = visitaForm.getHora_visita();
                String nombre_mascota = visitaForm.getNombre_mascota();
				String veterinario = visitaForm.getVeterinario();
				String centro_veterinario = visitaForm.getCentro_veterinario();
				Double peso = visitaForm.getPeso();
				String vacunas = visitaForm.getVacunas();
				String analisis = visitaForm.getAnalisis();
				String radiografias = visitaForm.getRadiografias();
				String exploracion = visitaForm.getExploracion();
				String otras_observaciones = visitaForm.getOtras_observaciones();
				Date proxima_visita = visitaForm.getProxima_visita();
				String hora_prox_visita = visitaForm.getHora_prox_visita();
        		
                visita.update(fecha_visita, hora_visita, nombre_mascota, veterinario, centro_veterinario,
                		peso, vacunas, analisis, radiografias,
                		exploracion, otras_observaciones, proxima_visita, hora_prox_visita);
                ofy().save().entity(visita).now();
                
                if (visita.getProxima_visita()!= null){
                	String nom_usuario = perfil.getNombre();
                    Date fecha = visita.getProxima_visita();
                    DateFormat formatoFecha = new SimpleDateFormat("dd-MM-yyyy");
                    DateFormat formatoFecha2 = new SimpleDateFormat("yyyyMMdd");
            		String dia = formatoFecha.format(fecha);
            		String dia_cita = formatoFecha2.format(fecha);
            		String fecha_cita;
            		String visitaInfo;
            		
            		if (visita.getHora_prox_visita() != null){
            			String hora = visita.getHora_prox_visita();
                		String hora2 = visita.getHora_prox_visita(); 
                		String hora3 = hora2.substring(0,2);
                		int horaNum = Integer.parseInt(hora3)+1;
                		String horaNew = null;
                		if (horaNum <=9){
                			horaNew = "0"+Integer.toString(horaNum);
                		}else{
                			horaNew = Integer.toString(horaNum);
                		}
                		hora2 = hora2.replace(hora3, horaNew);
                		String horaInicio = hora.replaceAll(":", "");
              			String horaFin = hora2.replaceAll(":", "");
              			visitaInfo="Cambio en la visita de "+nombre_mascota+".\n La próxima visita de "+nombre_mascota+ 
                				" al veterinario, será el día: "+dia+" a las: "+hora+"h.";
            			fecha_cita=dia_cita+"T"+horaInicio+"00/"+dia_cita+"T"+horaFin+"00";
            		}else{
            			visitaInfo="Cambio en la visita de "+nombre_mascota+".\n La próxima visita de "+nombre_mascota+ 
                				" al veterinario, será el día: "+dia+" pero aún no hemos concretado la hora.";
            			fecha_cita=dia_cita+"/"+dia_cita;
            		}
            		           		
            		String asunto_cita="Cita+veterniario+"+nombre_mascota;

                    String desc_cita="Siguiente+visita+de "+nombre_mascota;
                    
                    String centro_vet=visita.getCentro_veterinario();
                    
                    String html = "<h3>Próxima visita veterinaria</h3>"
                    		+"<p>Hola "+nom_usuario+", <br>"+visitaInfo+"</p>"
                    		+"<p>Puedes añadir esta cita a tu calendario de Google pulsando el siguiente botón.</p>"
                    		+"<a href='https://calendar.google.com/calendar/render?action=TEMPLATE&text="
                    		+asunto_cita+"&dates="+fecha_cita+"&details="+desc_cita+"&location="+centro_vet+"&trp=false'><img src='http://www.google.com/calendar/images/ext/gc_button6_es.gif' border='0'></a>"
                    		+"<p>Si agregaste la cita anterior a tu calendario, recuerda eliminarla.<p>";
            		
                    queue.add(ofy().getTransaction(),
                            TaskOptions.Builder.withUrl("/tasks/send_visit_email")
                            .param("email", perfil.getEmail()) 						
                            .param("mascota", nombre_mascota)
                    		.param("html_texto", html));
                }
                
                
                
                return new TxResult<>(visita);
            }
        });
        return result.getResult();
    }

     /**
      * Devuelve el objeto visita a través de su id
      * @param websafeVisitaKey
      * @return
      * @throws NotFoundException
      */
    @ApiMethod(
            name = "getVisita",
            path = "visita/{websafeVisitaKey}",
            httpMethod = HttpMethod.GET
    )
    public Visita getVisita(
            @Named("websafeVisitaKey") final String websafeVisitaKey)
            throws NotFoundException {
        Key<Visita> visitaKey = Key.create(websafeVisitaKey);
        Visita visita = ofy().load().key(visitaKey).now();
        if (visita == null) {
            throw new NotFoundException("No se encuentra la visita con la clave: " + websafeVisitaKey);
        }
        return visita;
    }

    /**
     * Devuelve las visitas del usuario
     * @param user
     * @return
     * @throws UnauthorizedException
     * @throws NotFoundException
     */
    @ApiMethod(
            name = "getVisitasDelUsuario",
            path = "getVisitasDelUsuario",
            httpMethod = HttpMethod.GET
    )
    public Collection<Visita> getVisitasDelUsuario(final User user)
            throws UnauthorizedException, NotFoundException {
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        Perfil perfil = ofy().load().key(Key.create(Perfil.class, getUserId(user))).now();
        if (perfil == null) {
            throw new NotFoundException("El perfil no existe.");
        }
        List<String> keyListaPropias = perfil.getVisitasPropiasKeys();
        List<Key<Visita>> keysPropias = new ArrayList<>();
        for (String keyString : keyListaPropias) {
        	keysPropias.add(Key.<Visita>create(keyString));
        }
        return ofy().load().keys(keysPropias).values();
    }

    /**
     * Consultar visitas del usuario
     * @param visitaConsultaForm
     * @param user
     * @return
     * @throws UnauthorizedException
     */
    @ApiMethod(
            name = "consultaVisitas",
            path = "consultaVisitas",
            httpMethod = HttpMethod.POST
    )
    public List<Visita> consultaVisitas(VisitaConsultaForm visitaConsultaForm, final User user) throws UnauthorizedException {
   
    	if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        String userId = getUserId(user);
        
        Query<Visita> consulta = visitaConsultaForm.getQuery(userId);
    
   	
        return consulta.list();
        
        
    }

     /**
      * Devuelve las visitas creadas por el usuario
      * @param user
      * @return
      * @throws UnauthorizedException
      */
    @ApiMethod(
            name = "getVisitasCreadas",
            path = "getVisitasCreadas",
            httpMethod = HttpMethod.POST
    )
    public List<Visita> getVisitasCreadas(final User user) throws UnauthorizedException {
        if (user == null) {
            throw new UnauthorizedException("Autorización requerida");
        }
        String userId = getUserId(user);
        
        
        return ofy().load().type(Visita.class)
                .ancestor(Key.create(Perfil.class, userId))
                .list();
    }

   

    /**
     * Elimina la visita
     * @param user
     * @param websafeVisitaKey
     * @return
     * @throws UnauthorizedException
     * @throws NotFoundException
     * @throws ForbiddenException
     * @throws ConflictException
     */
    @ApiMethod(
            name = "eliminarVisita",
            path = "visita/{websafeVisitaKey}/registros",
            httpMethod = HttpMethod.DELETE
    )
    public WrappedBoolean eliminarVisita(final User user,
                                            @Named("websafeVisitaKey")
                                            final String websafeVisitaKey)
            throws UnauthorizedException, NotFoundException, ForbiddenException, ConflictException {
        if (user == null) {
            throw new UnauthorizedException("Autorización Requerida");
        }
        final String userId = getUserId(user);
        TxResult<Boolean> result = ofy().transact(new Work<TxResult<Boolean>>() {

        	@Override
            public TxResult<Boolean> run() {
                Key<Visita> visitaKey = Key.create(websafeVisitaKey);
                Visita visita = ofy().load().key(visitaKey).now();
                  if (visita == null) {
                    return new TxResult<>(new NotFoundException(
                            "Visita con encontrada con la clave: " + websafeVisitaKey));
                }else{
                    ofy().delete().entity(visita).now();
                    return new TxResult<>(true);
                }
   
        	}   
        	});

	        return new WrappedBoolean(result.getResult());
    }    

}
